'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = build;

var _buffer = require('buffer');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _postcssCssnext = require('postcss-cssnext');

var _postcssCssnext2 = _interopRequireDefault(_postcssCssnext);

var _cssnano = require('cssnano');

var _cssnano2 = _interopRequireDefault(_cssnano);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _findBabelConfig = require('find-babel-config');

var _findBabelConfig2 = _interopRequireDefault(_findBabelConfig);

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

// Turn a UUID Buffer into a valid JS identifier.
function stringifyId(id) {
  return id.toString('base64').replace(/=+$/, '').replace(/\+/g, '$').replace(/\//g, '_');
}

function createWebpackConfig(options) {
  function postcssPlugins() {
    if (options.minify) {
      return [(0, _postcssCssnext2.default)({
        features: {
          autoprefixer: false
        }
      }), (0, _cssnano2.default)({
        zindex: false
      })];
    }
    return [(0, _postcssCssnext2.default)()];
  }

  var id = _buffer.Buffer.alloc(16);
  _uuid2.default.v4(null, id);

  var context = _path2.default.dirname(options.entry);

  var _findBabelConfig$sync = _findBabelConfig2.default.sync(context),
      config = _findBabelConfig$sync.config;

  if (config) {
    console.log(_chalk2.default.dim('Using custom Babel configuration.'));
  }

  return {
    context: context,
    entry: './' + _path2.default.basename(options.entry).replace(/\.js$/, ''),
    watch: !!options.watch,

    module: {
      rules: [{
        test: /\.json$/,
        use: require.resolve('json-loader')
      }, {
        test: /\.css$/,
        use: [{
          loader: require.resolve('css-loader'),
          options: { importLoaders: 1 }
        }, {
          loader: require.resolve('postcss-loader'),
          options: { plugins: postcssPlugins }
        }]
      }, {
        test: /\.js$/,
        exclude: /node_modules/,
        use: {
          loader: require.resolve('babel-loader'),
          options: config || { presets: [require.resolve('../babel')] }
        }
      }]
    },

    output: {
      path: _path2.default.dirname(options.output),
      filename: _path2.default.basename(options.output),
      jsonpFunction: 'jsonp' + stringifyId(id),
      libraryTarget: 'amd'
    },

    externals: ['jquery', 'underscore', 'backbone', 'plug-modules', 'meld', 'lang/Lang', function (_, request, cb) {
      if (/^plug\//.test(request)) {
        cb(null, 'amd plug-modules!' + request);
      } else if (/^extplug\//.test(request)) {
        cb(null, 'amd ' + request);
      } else {
        cb();
      }
    }],

    plugins: [options.minify && new _webpack2.default.optimize.UglifyJsPlugin()].filter(Boolean)
  };
}

function build(_ref) {
  var entry = _ref.entry,
      output = _ref.output,
      opts = _objectWithoutProperties(_ref, ['entry', 'output']);

  var config = createWebpackConfig(_extends({
    entry: entry,
    output: output
  }, opts));

  return new Promise(function (resolve, reject) {
    (0, _webpack2.default)(config).run(function (err, stats) {
      if (err) {
        reject([err]);
      } else if (stats.compilation.errors.length > 0) {
        reject(stats.compilation.errors);
      } else {
        resolve(stats);
      }
    });
  });
}
module.exports = exports['default'];